<?php

namespace FiloBlu\Refilo\Model\Exporter;

use FiloBlu\Refilo\Model\Adapter\Mongo;
use FiloBlu\Refilo\Model\Config\Seo\HrefLangConfig;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Store\Model\Service\StoreConfigManager;
use Magento\Store\Model\StoreManagerInterface;
use MageWorx\SeoBase\Helper\Hreflangs;
use Psr\Log\LoggerInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Class Urls
 * @package FiloBlu\Refilo\Model\Exporter
 */
class Urls extends AbstractExporter
{
    const XML_WEB_URL_REFILO_USE_STORE_IN_STOREFRONT = 'web/url/refilo_use_store_in_storefront';
    /**
     * @var Hreflangs
     */
    protected $hreflangHelper;
    /**
     * @var StoreConfigManager
     */
    protected $storeConfigManager;
    /**
     * @var array|null
     */
    protected $storeUrls;
    /**
     * @var array|null
     */
    protected $storeBaseUrls;
    /**
     * @var array|null
     */
    protected $storeLocales;
    /**
     * @var array|null
     */
    protected $seoBaseLocales;
    /**
     * @var string | null
     */
    protected $homeIdentifier;
    /**
     * @var HrefLangConfig
     */
    protected $hrefLangConfig;
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;
    /**
     * @var LoggerInterface
     */
    private $logger;
    /**
     * @var array
     */
    private $hrefLangConfigLookup;
    /**
     * @var \FiloBlu\Refilo\Helper\Data
     */
    private $helper;
    /**
     * @var
     */
    private $homeUrls = [];

    /**
     * Urls constructor.
     * @param ResourceConnection $resourceConnection
     * @param ScopeConfigInterface $scopeConfig
     * @param Mongo $mongo
     * @param StoreManagerInterface $storeManager
     * @param StoreConfigManager $storeConfigManager
     * @param Hreflangs $hreflangs
     * @param LoggerInterface $logger
     * @param HrefLangConfig $hrefLangConfig
     * @param \FiloBlu\Refilo\Helper\Data $helper
     */
    public function __construct(
        ResourceConnection $resourceConnection,
        ScopeConfigInterface $scopeConfig,
        Mongo $mongo,
        StoreManagerInterface $storeManager,
        StoreConfigManager $storeConfigManager,
        Hreflangs $hreflangs,
        LoggerInterface $logger,
        HrefLangConfig $hrefLangConfig,
        \FiloBlu\Refilo\Helper\Data $helper
    )
    {
        parent::__construct($mongo, $scopeConfig);
        $this->resourceConnection = $resourceConnection;
        $this->storeManager = $storeManager;
        $this->storeConfigManager = $storeConfigManager;
        $this->hreflangHelper = $hreflangs;
        $this->logger = $logger;
        $this->hrefLangConfig = $hrefLangConfig;
        $this->helper = $helper;
    }

    public function getCollections(): iterable
    {
        yield 'urls';
    }

    /**
     * @param OutputInterface|null $output
     * @throws \MongoDB\Driver\Exception\Exception
     */
    public function execute(OutputInterface $output = null)
    {
        $entities = $this->init();
        foreach ($entities as $type => $ids) {
            $max = count($ids);
            $pages = 10;
            $i = 0;
            while ($i <= $max) {
                $this->exportUrls($this->getUrls(array_slice($ids, $i, $pages), $type), $output);
                $i += $pages;
            }
        }
    }

    /**
     * Init Function
     */
    protected function init()
    {
        $entities = [];
        $tableName = $this->resourceConnection->getTableName('url_rewrite');
        $query_type = "SELECT DISTINCT {$tableName}.entity_type FROM {$tableName}";
        $entity_type = $this->resourceConnection->getConnection()->fetchCol($query_type, []) ?? [];

        foreach ($entity_type as $type) {
            $query = "SELECT DISTINCT {$tableName}.entity_id FROM {$tableName} WHERE entity_type = '{$type}'";
            $entities[$type] = $this->resourceConnection->getConnection()->fetchCol($query, []) ?? [];
        }

        $this->seoBaseLocales = [];
        $this->storeUrls = [];
        $this->storeBaseUrls = [];
        $this->hrefLangConfigLookup = [];
        $locale = '';
        $addCountryCode = false;

        foreach ($this->storeManager->getStores() as $store) {
            $storeId = (int)$store->getId();
            $this->storeUrls[$storeId] = $this->helper->getFrontendUrl($store);
            $this->storeBaseUrls[$storeId] = $this->helper->getStoreFrontBaseUrl($store);
            $this->seoBaseLocales[$store->getCode()] = $this->hrefLangConfig->getSeoLocaleCodeForStore($store);
            $this->hrefLangConfigLookup[$storeId] = $this->hrefLangConfig->getConfig($store);
        }

        $this->storeLocales = [];
        foreach ($this->storeConfigManager->getStoreConfigs() as $storeConfig) {
            $this->storeLocales[$storeConfig->getCode()] = $storeConfig->getLocale();
        }

        $this->homeIdentifier = $this->scopeConfig->getValue('web/default/cms_home_page');

        return $entities;
    }

    /**
     * @throws \MongoDB\Driver\Exception\Exception
     */
    public function exportUrls($result, OutputInterface $output = null)
    {
        $lookupRewrites = [];
        $bulkData = [];

        foreach ($result as $item) {
            $targetPath = $item['target_path'];
            if($item['entity_type']==='cms-page'){
                $targetPath = $item['refilo_page_identifier'];
            }

            if (!isset($lookupRewrites[$targetPath])) {
                $lookupRewrites[$targetPath] = [];
            }

            $lookupRewrites[$targetPath][] = $item;
        }

        foreach ($result as $item) {
            $bulkData[] = [
                '_id'         => $item['url_rewrite_id'],
                'url_id'      => $item['url_rewrite_id'],
                'type'        => $item['entity_type'] === 'contenttype_content' ? 'master_data' : $item['entity_type'],
                'entity_id'   => $item['entity_id'],
                'url'         => $item['request_path'],
                'is_redirect' => $item['redirect_type'],
                'target_url'  => $item['target_path'],
                'store'       => $item['store_code'],
                'hreflang'    => $this->getHreflang(($item['entity_type']==='cms-page')?$item['refilo_page_identifier']:$item['target_path'], $item['request_path'], $item['entity_type'], $lookupRewrites)
            ];
        }

        if (!empty($bulkData)) {
            try {
               $this->getConnection()->update('_id', $bulkData, 'urls', true, $this->getDatabase());
               $this->getConnection()->ensureIndex(['url'], 'urls');
            } catch (\Throwable $throwable) {
                $this->logger->error($throwable->getMessage(), ['exception' => $throwable]);
                if ($output) {
                    $output->writeln($throwable->getMessage());
                }
            }

        }
    }

    /**
     * @param $targetPath
     * @param $requestPath
     * @param $type
     * @param $result
     * @return array
     */
    private function getHreflang($targetPath, $requestPath, $type, $result)
    {
        $hrefArray = [];
        $isHomePage = $requestPath === $this->homeIdentifier;

        if(empty($targetPath)){
            return [];
        }

        foreach ($result[$targetPath] as $url) {
            //If Disabled General HrefLang
            if (!$this->hrefLangConfigLookup[(int)$url['store_id']]['enabled']) {
                return [];
            }
            //If Disabled Cms HrefLang
            if (!$this->hrefLangConfigLookup[(int)$url['store_id']]['enabled_cms'] && $type === 'cms-page' && !$isHomePage) {
                return [];
            }
            //If Disabled Home HrefLang
            if (!$this->hrefLangConfigLookup[(int)$url['store_id']]['enabled_home'] && $type === 'cms-page' && $isHomePage) {
                return [];
            }
            //If Disabled Category HrefLang
            if (!$this->hrefLangConfigLookup[(int)$url['store_id']]['enabled_category'] && $type === 'category') {
                return [];
            }
            //If Disabled Product HrefLang
            if (!$this->hrefLangConfigLookup[(int)$url['store_id']]['enabled_product'] && $type === 'product') {
                return [];
            }

            if (!isset($url['store_code'], $url['store_id']) || (int)$url['store_id'] === 0) {
                continue;
            }

            if ($isHomePage && $type === 'cms-page') {
                $hrefArray['x-default'] = [
                    'code' => 'x-default',
                    'url'  => $this->storeBaseUrls[$url['store_id']]
                ];
                $hrefArray = array_merge($hrefArray,$this->getHomeUrls());
            }

            if (!$isHomePage && $url['target_path'] === $targetPath) {
                $hrefArray[$url['store_code']] = [
                    'code' => ($this->hrefLangConfigLookup[(int)$url['store_id']]['use_magento_lang_code']) ? $this->storeLocales[$url['store_code']] : $this->seoBaseLocales[$url['store_code']],
                    'url'  => "{$this->storeUrls[$url['store_id']]}{$url['request_path']}"
                ];
            }

            if (!$isHomePage && $type === 'cms-page' && $url['refilo_page_identifier'] === $targetPath) {
                $hrefArray[$url['store_code']] = [
                    'code' => ($this->hrefLangConfigLookup[(int)$url['store_id']]['use_magento_lang_code']) ? $this->storeLocales[$url['store_code']] : $this->seoBaseLocales[$url['store_code']],
                    'url'  => "{$this->storeUrls[$url['store_id']]}{$url['request_path']}"
                ];
            }
        }

        return $hrefArray;
    }

    /**
     * @param $ids
     * @param $entityType
     * @return array
     */
    public function getUrls($ids, $entityType)
    {
        if(empty($ids) || !$entityType) {
            return [];
        }

        $ids = implode(",", $ids);
        $tableName = $this->resourceConnection->getTableName('url_rewrite');
        if($entityType==="cms-page"){
            $cmsTableName = $this->resourceConnection->getTableName('cms_page');
            $query = "SELECT {$tableName}.*,store.code as store_code, cp.refilo_page_identifier FROM {$tableName} INNER JOIN store on store.store_id = {$tableName}.store_id INNER JOIN {$cmsTableName} cp ON url_rewrite.entity_id = cp.page_id WHERE {$tableName}.store_id != 0 AND entity_type = '{$entityType}' AND entity_id IN ({$ids})";
        }
        else {
            $query = "SELECT {$tableName}.*,store.code as store_code FROM {$tableName} INNER JOIN store on store.store_id = {$tableName}.store_id WHERE {$tableName}.store_id != 0 AND entity_type = '{$entityType}' AND entity_id IN ({$ids})";
        }

        return $this->resourceConnection->getConnection()->fetchAll($query, [], \Zend_Db::FETCH_ASSOC) ?? [];
    }

    /**
     * @return array
     */
    public function getHomeUrls() {
        if(count($this->homeUrls)>0){
            return $this->homeUrls;
        }
        $tableName = $this->resourceConnection->getTableName('url_rewrite');
        $query = "SELECT {$tableName}.*, store.code as store_code FROM {$tableName} INNER JOIN store on store.store_id = {$tableName}.store_id WHERE {$tableName}.store_id != 0 AND request_path = '{$this->homeIdentifier}'";
        $urls = $this->resourceConnection->getConnection()->fetchAll($query, [], \Zend_Db::FETCH_ASSOC) ?? [];

        if(count($urls)>0) {
            foreach ($urls as $url){
                $this->homeUrls[$url['store_code']] = [
                    'code' => ($this->hrefLangConfigLookup[(int)$url['store_id']]['use_magento_lang_code']) ? $this->storeLocales[$url['store_code']] : $this->seoBaseLocales[$url['store_code']],
                    'url'  => $this->storeUrls[$url['store_id']]
                ];
            }
        }
        return $this->homeUrls;
    }
}
